% This is the code for computing the equilibrium allocaiton
% of a standard NK with binding zlb

clear all;
close all;
loc = pwd;
addpath(genpath(loc));
dbstop if error

%you have the option of computing an equilibrium using the following
%parameter values, or of loading a previously computed equilibrium and
%plotting the information about that.

previous=0;

if previous ~= 1
    
    lump=0;%if 1, then use lump sum taxes to satisfy intertemporal budget constraint.
    % Parameteres (quarterly)
    epsilon     = 6; % elasticity of the final good producer's production function
    beta        =.99; % discount factor
    
    % Taylor
    rhoR    = 0;% Taylor rule's backwardness
    phipi   = 1.5; % responsiveness to the inflaiton
    phic    = 0; % does not respond to the output gap
    
    % Calvo adjustment parameter
    %theta   = .75;
    theta   = 0.844956044718208;
    %theta=0.82;
    n_steady= 0.33;
    
    % utility preference
    util_type = 'KPR';
    if strcmp(util_type,'separable')
        sigma   = 1; % log utility function
        psi     = 1; % Frisch elasticity
    else
        sigma = 2;
        psi=1;%this gets overwritten by a value compatible with n_steady
    end
    
    
    % liquidity trap
    rh      = 1/beta-1;
    rl     = -.02/4; % negative natural rate
    %rl      = rh; % mute the discount rate shock if you want.
    tp      = 12; % tp is the last day when beta is equal to beta_low;
    det_    = 1;%if 1, r = rl for t=1...tp, = rh for t=tp+1,...,T, deterministically.
                %if 0, then start at r=rl and jump with constant probability.
                %system is assumed to be in steady state in T+1
    T   = tp + 400;
    
    % Policy related parameters
    g=.2;
    eta = .05; % responsiveness of debt
    omega1 = .95;
    %omega1 = 0;
    omega2 = 0;
    % calibration target
    rb = 1/2; % rb = b_g_ss/output
    tau_ss = 1/3;
    
    
    % Simulated path for the technology : a_path
    % a(t+1) = (rho1 + rho2) * a(t) - rho1 * rho2 * a(t-1) + epsi.
    rho1        = .4;
    %rho2        = .7;
    rho2       = .0;
    epsi        = -.01;
    epsi =0;
    
    % Simulated path for the lump-sum tax : t_g_path (if lump == 1, then
    % parameters used for law of motion for tau, labor tax rate
    % t_g(t+1) = t_g(t) ^ (rho1 + rho2)  * t_g(t-1)^(- rho1 * rho2) * exp(epsi).
    rho_t_g1        = 0.8;
    rho_t_g2       = .0;
    epsi_t        =   -.2;
    epsi_t = 0;
    % Simulated path for the lump-sum tax : t_g_path
    % G(t+1) = G(t) ^ (rho1 + rho2)  * G(t-1)^(- rho1 * rho2) * exp(epsi).
    rho_G1        = 0.8;
    rho_G2       = .0;
    epsi_G        = .2;
        
    % Construct the exogenous paths
    
    a_path      = nan(tp,1);
    a_path(1)   = epsi;
    a_path(2)   = (rho1 + rho2) * a_path(1);
    for i = 3 : tp
        a_path(i) = (rho1 + rho2) * a_path(i-1) -rho1*rho2 * a_path(i-2);
    end
    
    
    % check that tp is sufficiently big
    dist = norm(a_path(end));
    if dist>1e-4
        error('Choose a bigger tp');
    end
    dt  = [1 : T]';
    
    % recompute the path
    for i = tp+1 : T
        a_path(i) = (rho1 + rho2) * a_path(i-1) -rho1*rho2 * a_path(i-2);
    end
    a_path_impulse = a_path;
    % interest rate path (forecasted)
    if det_~=1
        r_path = q.^dt * rl  + (1-q.^dt) *rh;
    else
        r_path(1:tp)=rl;
        r_path(tp+1:T)=rh;
    end
    
    
    % Compute the steady state relationship
    
    [c_ss p_ss n_ss pi_ss K_ss F_ss R_ss w_ss psi] =ss_ZLB(beta,theta,g,psi,util_type,n_steady);
    
    % NOTE
    % if util_type is KPR, then psi is the parameters for substitution between
    % C and N.
    % if util_type is separable, psi returns the original psi.
    
    
    % Compute the new faces.
    % b_g_ss : steady state bond supply
    % tau_ss : the steady state labor tax
    % nu     : the steady state subsidy to firms which is designed to eliminate
    % the steady state distortions
    %
    
    b_g_ss = rb*n_ss;
    Omega_ss = b_g_ss;
    % steady state subsidy (designed for eliminating the steady state distortion)
    nu = 1-(1-tau_ss)*(epsilon-1)/epsilon;
    t_g_ss = ( rb*(1/beta-1) + (nu-tau_ss)/(1-tau_ss) + g ) *n_ss;    
    Zss     = [c_ss p_ss n_ss pi_ss K_ss F_ss R_ss b_g_ss tau_ss Omega_ss t_g_ss]';
    N       = length(Zss)-1; % number of the endogenous variables
    if lump == 1;N=N-3;end%in lump sum case, drop last three variables
    
    
    % Compute the exogenous shocks
    if lump == 0
        t_g_path      = nan(T,1);
        t_g_path(1)   = t_g_ss * exp(epsi_t);
        t_g_path(2)   = t_g_ss^( (1-rho_t_g1 ) * ( 1-rho_t_g2 )   ) * t_g_path(1)^((rho_t_g1 + rho_t_g2));
        for i = 3 : T
            t_g_path(i) =  t_g_ss^( (1-rho_t_g1 ) * ( 1-rho_t_g2 )   ) * t_g_path(i-1)^(rho_t_g1 + rho_t_g2 )  * t_g_path(i-2)^(-rho_t_g1*rho_t_g2);
        end
        tau_path=[];
    else
        tau_path      = nan(T,1);
        tau_path(1)   = tau_ss * exp(epsi_t);
        tau_path(2)   = tau_ss^( (1-rho_t_g1 ) * ( 1-rho_t_g2 )   ) * tau_path(1)^((rho_t_g1 + rho_t_g2));
        for i = 3 : T
            tau_path(i) =  tau_ss^( (1-rho_t_g1 ) * ( 1-rho_t_g2 )   ) * tau_path(i-1)^(rho_t_g1 + rho_t_g2 )  * tau_path(i-2)^(-rho_t_g1*rho_t_g2);
        end
        t_g_path=[];
    end
    
    G_ss = g*n_ss;
    G_path      = nan(T,1);
    G_path(1)   = G_ss * exp(epsi_G);
    G_path(2)   = G_ss^( (1-rho_G1 ) * ( 1-rho_G2 )   )* G_path(1)^((rho_G1 + rho_G2));
    for i = 3 : T
        G_path(i) =  G_ss^( (1-rho_G1 ) * ( 1-rho_G2 )   ) * G_path(i-1)^(rho_G1 + rho_G2 )  * G_path(i-2)^(-rho_G1*rho_G2);
    end
    G_impulse = log(G_path/G_ss);
    
    % Solving the model
    % Set the optionTolX
    options             = optimset('Display','iter','MaxFunEvals',1000000,'TolX',1e-12,'TolFun',1e-12,'MaxIter',1000);
    III=[1 2 3 4 5 6 7 8 9 10];
    if lump == 1
        III=[1 2 3 4 5 6 7];%in lump sum tax case (lump==1) drop last three variables
    end
    
    init=1;%if unity use previous solution, otherwise go with steady state.
    if lump== 1
        fname='results_baseline_lump';
    else
        fname='results_theta82';
    end
    [Z_initial_guess,Z_initial_without_guess] = get_started(init,Zss(III),T,lump,fname);
    
    
    % Prepare the box
    Z_forecast = cell(T,1); % Agent's forecast.
    Z_actual   = nan(N,T);  % The acual realization
    
    %Simulation with impulse 
    tic;stochastic_simulation_distortionary_tax;toc
    Z_impulse = Z_forecast{1};
    Y_impulse=Z_impulse(1,:)'+G_path;    
    
    %the matrix A is computed and stored so that the results can be compared
    %with the linear solution in /Users/christiano/Dropbox/sw20_LJC/docs/Teaching/Monetary/Korea_2014/fiscal_policy/Dynare_code
    if lump == 1
        Z_impulse(8,:)=Zss(8)*ones(1,T);
        Z_impulse(9,:)=tau_path';
        Z_impulse(10,:)=Zss(8)*ones(1,T);
        gamma=psi;
        mrs=((1-gamma)/gamma)*Z_impulse(1,:)./(1-Z_impulse(3,:));
        Rm1=[Zss(7) Z_impulse(7,1:end-1)];
        labor_income=((tau_path'-nu).*(mrs./(1-tau_path')).*Z_impulse(3,:));
        real_r=(Rm1./Z_impulse(4,:));
        t_g_path=G_path'-labor_income-Zss(8)+real_r*Zss(8);
        t_g_path=t_g_path';
    end
    %non-linear code: C,p*,N, pie,K,F,R,b+1,tau,om
    %linearization code: {'y','inflation','R','tau','g','tg','Omega','Real_Rate','bg','labor_tax_revenues'}
    %rows of A correspond to ordering in linearization code
    A(1,:)=Z_impulse(3,:).*Z_impulse(2,:);
    A(2,:)=Z_impulse(4,:);
    A(3,:)=Z_impulse(7,:);
    A(4,:)=Z_impulse(9,:);
    A(5,:)=G_path';
    A(6,:)=t_g_path';   
    A(7,:)=Z_impulse(10,:);
    A(8,:)=[Z_impulse(7,:)]./[Z_impulse(4,2:end) 1];
    A(9,:)=Z_impulse(8,:);
    A(10,:)=Z_impulse(2,:);
    %
    save impes_nonlinear A
    
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % Simulation without impulse (without technology shock, but with the ZLB)
    % Reset
    Z_forecast          = cell(T,1); % Agent's forecast.
    Z_actual            = nan(N,T);  % The acual realization
    Z_initial_guess     = repmat(Zss(III),T,1); 
    if lump == 1
        B                   = Z_impulse(1:7,:);
        Z_initial_guess     = B(:);%initial guess is solution to previous run
    else
        Z_initial_guess = Z_impulse(:);
    end
    G_path              = ones(size(G_path))*G_ss;
    
    if det_ ~= 1
        r_path              = q.^dt * rl  + (1-q.^dt) *rh;
    end
    
    % Do the stochastic simulation
    Z_initial_guess=Z_initial_without_guess;
    tic;stochastic_simulation_distortionary_tax;toc
    Z_without_impulse = Z_forecast{1};
    %non-linear code: C,p*,N, pie,K,F,R,b+1,tau,om
    if lump == 1
        Z_without_impulse(8,:)=Zss(8)*ones(1,T);
        Z_without_impulse(9,:)=tau_path';
        Z_without_impulse(10,:)=Zss(8)*ones(1,T);
        gamma=psi;
        mrs=((1-gamma)/gamma)*Z_without_impulse(1,:)./(1-Z_without_impulse(3,:));
        Rm1=[Zss(7) Z_without_impulse(7,1:end-1)];
        t_g_path=G_path'-((tau_path'-nu).*mrs./(1-tau_path').*Z_without_impulse(3,:)) ...
            -Zss(8)+(Rm1./Z_without_impulse(4,:))*Zss(8);
        t_g_path=t_g_path';
    end
    
    save results
    
else
    
    load results
    
end


s1=['parameter values: \phi_{1} = ',num2str(phipi),', \phi_{2} = 0, \rho_{R} = ',num2str(rhoR) ...
        ,', \rho_{1} = ',num2str(rho_t_g1),', \rho_{2} = ',num2str(rho_t_g2), ...
        ', \kappa = ',num2str((1-theta)*(1-beta*theta)/theta), ...
        ', \beta = ',num2str(beta),', lump = ',num2str(lump) ...
        ,', \gamma = ',num2str(psi),', N = ',num2str(n_steady),', G/Y = ',num2str(g), ...
        ', bg = ',num2str(b_g_ss),', nu = ',num2str(nu),', rh = ',num2str(rh),', rl = ',num2str(rl),', tp = ',num2str(tp),', T = ',num2str(T)];
s2=['  sig (utility) = ',num2str(sigma),',\eta = ',num2str(eta),',\epsilon = ',num2str(epsilon), ...
        ', r^b = ',num2str(rb), ...
        ',\nu = ',num2str(nu),', t^g = ',num2str(t_g_ss),', \omega_{1} = ',num2str(omega1), ...
        ', \omega_{2} = ',num2str(omega2),', theta = ',num2str(theta), ', utility function: ',util_type, ...
        ', rho(G)_{1} = ',num2str(rho_G1),', rho(G)_{2} = ',num2str(rho_G2),', std(G innovation) = ',num2str(epsi_G)];

    
% Statistics
plt_nonl(Z_impulse,Z_without_impulse,G_path,t_g_path,A,Zss,psi,sigma,nu,omega1,eta,r_path,beta, ...
    theta,epsilon,sigma,s1,s2,g,lump);
1+1;

 